using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;

namespace DetexGUI
{
    public partial class detexGUIForm : Form
    {
        private string inputFile;
        private const string errorCaption = "Detex Error";
        private const string warningCaption = "Detex Warning";

        private string errorFile;
        private string latexFile;

        private string workingDir;

        public detexGUIForm()
        {
            InitializeComponent();

            //Disable and enable checkboxes
            outputFileEnableDisable();
        }

        private void deleteTempFiles()
        {
            //Error file
            try
            {
                FileInfo errFile = new FileInfo(errorFile);
                errFile.Delete();
            }
            catch 
            {
                //Fail silently
            }

            //Latex file
            try
            {
                FileInfo detexFile = new FileInfo(latexFile);
                detexFile.Delete();
            }
            catch 
            {
                //Fail silently
            }
        }

        private void outputFileEnableDisable()
        {
           if (saveFileCheckBox.Checked)
            {
                outputFileBox.Enabled = true;
                outputFileButton.Enabled = true;
            }
            else
            {
                outputFileBox.Enabled = false;
                outputFileButton.Enabled = false;
            }
        }


        private void resetDetexOutput()
        {
            wordCountBox.Text = "";
            displayDetex.Text = "";
        }

        private void readErrorFile()
        {
            //Read error file

            try
            {
                FileInfo errFile = new FileInfo(errorFile);
                StreamReader errStream = errFile.OpenText();

                String text = "";
                String tmpText;

                bool warningsExist = false;

                do
                {
                    tmpText = errStream.ReadLine();
                    if (tmpText != null)
                    {
                        text += tmpText;
                        text += Environment.NewLine;
                        warningsExist = true;
                    }
                } while (tmpText != null);

                if (warningsExist)
                {
                    MessageBox.Show(text, warningCaption, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
             
                //close files
                errStream.Close();
            }
            catch 
            {
                //Fail silently
            }
        }

        private void detexButton_Click(object sender, EventArgs e)
        {
            //Make sure inputFile is up to date with the GUI openFileNameBox
            inputFile = inputFileBox.Text;

            //Check that a file is selected
           
            if(inputFile == "")
            {
                String error_string = "No file selected";
                MessageBox.Show(error_string, errorCaption, MessageBoxButtons.OK, MessageBoxIcon.Error);

                resetDetexOutput();

                return;
            }
            
            //Call the detex DLL
            int result = DetexDLL.detex_dll_main(inputFile, latexFile, errorFile);
            
            //Success
            if (result == 0)
            {
                //Read and display detexed text

                string text = "";

                try
                {
                    FileInfo detexFile = new FileInfo(latexFile);

                    StreamReader inputStream = detexFile.OpenText();

                    string tmpText;
                    do
                    {
                        tmpText = inputStream.ReadLine();
                        if (tmpText != null)
                        {
                            text += tmpText;
                            text += Environment.NewLine;
                        }
                    } while (tmpText != null);

                    displayDetex.Text = text;
                    displayDetex.ReadOnly = true;

                    inputStream.Close();
                }
                catch 
                {
                    MessageBox.Show("Error reading _latex.tmp. Please check this file is not open in another program", errorCaption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                //Do word count

                wordCountBox.Text = (WordCount.CountWords(text)).ToString();
                wordCountBox.ReadOnly = true;

                //Read error file if it exists
                readErrorFile();

                //Copy to clipboard if required

                if (copyCheckBox.Checked)
                {
                    try
                    {
                        Clipboard.SetData(System.Windows.Forms.DataFormats.Text, text); //exceptions?
                    }
                    catch 
                    {
                        MessageBox.Show("Error writing to clipboard", errorCaption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }

                //Save to file if required
                if (saveFileCheckBox.Checked)
                {
                    //Skip if we have no filename
                    if (outputFileBox.Text != "")
                    {
                        try
                        {
                            StreamWriter outputStream = new StreamWriter(outputFileBox.Text);
                            outputStream.Write(text);
                            outputStream.Close();
                        }
                        catch 
                        {
                            string errorMsg = "Error writing to: ";
                            errorMsg += outputFileBox.Text;
                            MessageBox.Show(errorMsg, errorCaption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                    else
                    {
                        MessageBox.Show("No output filename selected", errorCaption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
            
            //File not found
            else if(result == 1)
            {
                String error_string = "File not found: ";
                error_string += inputFile;

                MessageBox.Show(error_string, errorCaption, MessageBoxButtons.OK, MessageBoxIcon.Error);

                resetDetexOutput();
            }

            //Unable to create temporary file
            else if (result == 2)
            {
                String error_string = "Unable to create temporary file: _latex.tmp";

                MessageBox.Show(error_string, errorCaption, MessageBoxButtons.OK, MessageBoxIcon.Error);

                resetDetexOutput();
            }
            
            //Delete temporary files
            deleteTempFiles();
        }

        private void inputFileButton_Click(object sender, EventArgs e)
        {
            //Setup dialog box

            inputFileOpen.Filter = "Latex files (*.tex)|*.tex|All files (*.*)|*.*";
            inputFileOpen.Title = "Open latex file";
            inputFileOpen.FileName = "";

            DialogResult res = inputFileOpen.ShowDialog();
            if (res == DialogResult.OK)
            {
                inputFileBox.Text = inputFileOpen.FileName;
                inputFile = inputFileOpen.FileName;

                //If outputFileBox is empty, fill it with a detexed file name
                if (outputFileBox.Text == "")
                {
                    int dotIndex = inputFile.LastIndexOf(".");
                    string firstPart = inputFile.Substring(0, dotIndex);
                    firstPart += "-detexed.txt";

                    outputFileBox.Text = firstPart;
                }
            }
        }

        private void outputFileButton_Click(object sender, EventArgs e)
        {
            //Do nothing if saveFileBox is unchecked
            if (!saveFileCheckBox.Checked)
            {
                return;
            }

            //Setup file dialog box

            outputFileSave.Filter = "Text files (*.txt)|*.txt|All files (*.*)|*.*";
            outputFileSave.Title = "Save detexed file";
            outputFileSave.FileName = outputFileBox.Text;
            outputFileSave.ValidateNames = true;
            
            DialogResult res = outputFileSave.ShowDialog();
            if (res == DialogResult.OK)
            {
                outputFileBox.Text = outputFileSave.FileName;
            }
        }

        private void saveFileCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            //Enable or disable the outputFileBox as appropriate
            outputFileEnableDisable();
        }

        private void detexGUIForm_Load(object sender, EventArgs e)
        {
            // Set up the delays for the ToolTip.
            toolTip1.AutoPopDelay = 5000;
            toolTip1.InitialDelay = 1000;
            toolTip1.ReshowDelay = 500;
         
            // Set up the ToolTip text for the Button and Checkbox.
            toolTip1.SetToolTip(inputFileButton, "Choose the file to be detexed");
            toolTip1.SetToolTip(inputFileBox, "Choose the file to be detexed");
            toolTip1.SetToolTip(detexButton, "Run detex on the selected file");
            toolTip1.SetToolTip(copyCheckBox, "Copy the detex output to the Windows clipboard");
            toolTip1.SetToolTip(saveFileCheckBox, "Save the detex output to the file selected below");
            toolTip1.SetToolTip(outputFileBox, "Choose the file for the detex output");
            toolTip1.SetToolTip(outputFileButton, "Choose the file for the detex output");
            toolTip1.SetToolTip(wordCountBox, "The number of words in the detex output (note that detex output contains captions etc.)");

            //Set working directory
            workingDir = "";
            
            try {
                workingDir = Directory.GetCurrentDirectory();
            }
            catch {
                //This is bad
                MessageBox.Show("Unable to get current directory - program may not work as expected", errorCaption, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            //Set the names of the latex and error files
            latexFile = workingDir + @"\_latex.tmp";
            errorFile = workingDir + @"\_stderr.tmp";

            //Read configuration in config.XML
            try
            {
                XmlReader reader = XmlReader.Create("config.xml");

                try
                {
                    reader.ReadStartElement("root");

                    reader.ReadStartElement("Copy");
                    string copyString = reader.ReadString();
                    reader.ReadEndElement();

                    reader.ReadStartElement("Save");
                    string saveString = reader.ReadString();
                    reader.ReadEndElement();

                    if (copyString == "Y")
                    {
                        copyCheckBox.Checked = true;
                    }
                    else
                    {
                        copyCheckBox.Checked = false;
                    }

                    if (saveString == "Y")
                    {
                        saveFileCheckBox.Checked = true;
                    }
                    else
                    {
                        saveFileCheckBox.Checked = false;
                    }
                }
                catch
                {

                }
                finally
                {
                    reader.Close();
                }
            }
            catch 
            {
                MessageBox.Show("Cannot open configuration file: config.xml", warningCaption, MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }

        }

        private void detexGUIForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            try {
                string configTarget = workingDir + @"\config.xml";
                XmlWriter writer = XmlWriter.Create(configTarget);

                writer.WriteStartDocument();
                writer.WriteStartElement("root");

                if (copyCheckBox.Checked == true)
                {
                    writer.WriteElementString("Copy", "Y");
                }
                else
                {
                    writer.WriteElementString("Copy", "N");
                }

                if (saveFileCheckBox.Checked == true)
                {
                    writer.WriteElementString("Save", "Y");
                }
                else
                {
                    writer.WriteElementString("Save", "N");
                }

                writer.WriteEndElement();

                writer.Close();
            }
            catch {

                MessageBox.Show("Cannot save configuration file: config.xml", warningCaption, MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }
     }
}